@@ -21,7 +21,7 @@
 
 #include <config.h>
 #include "djvu-document.h"
-#include "djvu-text.h"
+#include "djvu-text-page.h"
 #include "djvu-links.h"
 #include "djvu-document-private.h"
 #include "ev-document-thumbnails.h"
@@ -60,16 +60,59 @@ static void djvu_selection_iface_init (EvSelectionIface *iface);
 
 EV_BACKEND_REGISTER_WITH_CODE (DjvuDocument, djvu_document,
     {
-      G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_THUMBNAILS, djvu_document_document_thumbnails_iface_init);
-      G_IMPLEMENT_INTERFACE (EV_TYPE_FILE_EXPORTER, djvu_document_file_exporter_iface_init);
-      G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_FIND, djvu_document_find_iface_init);
-      G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_LINKS, djvu_document_document_links_iface_init);
-      G_IMPLEMENT_INTERFACE (EV_TYPE_SELECTION, djvu_selection_iface_init);
+      EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_THUMBNAILS, djvu_document_document_thumbnails_iface_init);
+      EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_FILE_EXPORTER, djvu_document_file_exporter_iface_init);
+      EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_FIND, djvu_document_find_iface_init);
+      EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_LINKS, djvu_document_document_links_iface_init);
+      EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_SELECTION, djvu_selection_iface_init);
      });
 
 
-void 
-djvu_handle_events (DjvuDocument *djvu_document, int wait)
+#define EV_DJVU_ERROR ev_djvu_error_quark ()
+
+static GQuark
+ev_djvu_error_quark (void)
+{
+	static GQuark q = 0;
+	if (q == 0)
+		q = g_quark_from_static_string ("ev-djvu-quark");
+	
+	return q;
+}
+
+static void
+handle_message (const ddjvu_message_t *msg, GError **error)
+{
+	switch (msg->m_any.tag) {
+	        case DDJVU_ERROR: {
+			gchar *error_str;
+			
+			if (msg->m_error.filename) {
+				error_str = g_strdup_printf ("DjvuLibre error: %s:%d",
+							     msg->m_error.filename,
+							     msg->m_error.lineno);
+			} else {
+				error_str = g_strdup_printf ("DjvuLibre error: %s",
+							     msg->m_error.message);
+			}
+			
+			if (error) {
+				g_set_error_literal (error, EV_DJVU_ERROR, 0, error_str);
+			} else {
+				g_warning ("%s", error_str);
+			}
+				
+			g_free (error_str);
+			return;
+			}						     
+			break;
+	        default:
+			break;
+	}
+}
+
+void
+djvu_handle_events (DjvuDocument *djvu_document, int wait, GError **error)
 {
 	ddjvu_context_t *ctx = djvu_document->d_context;
 	const ddjvu_message_t *msg;
@@ -78,25 +121,33 @@ djvu_handle_events (DjvuDocument *djvu_document, int wait)
 		return;
 
 	if (wait)
-		msg = ddjvu_message_wait (ctx);
+		ddjvu_message_wait (ctx);
 
 	while ((msg = ddjvu_message_peek (ctx))) {
-		switch (msg->m_any.tag) {
-			case DDJVU_ERROR:
-				g_warning ("DjvuLibre error: %s", 
-					   msg->m_error.message);
-				if (msg->m_error.filename)
-					g_warning ("DjvuLibre error: %s:%d", 
-						   msg->m_error.filename,
-						   msg->m_error.lineno);
-				break;
-			default:
-				break;
-		}
+		handle_message (msg, error);
 		ddjvu_message_pop (ctx);
+		if (error && *error)
+			return;
 	}
 }
 
+static void
+djvu_wait_for_message (DjvuDocument *djvu_document, ddjvu_message_tag_t message, GError **error)
+{
+	ddjvu_context_t *ctx = djvu_document->d_context;
+	const ddjvu_message_t *msg;
+
+	ddjvu_message_wait (ctx);
+	while ((msg = ddjvu_message_peek (ctx)) && (msg->m_any.tag != message)) {
+		handle_message (msg, error);
+		ddjvu_message_pop (ctx);
+		if (error && *error)
+			return;
+	}
+	if (msg && msg->m_any.tag == message)
+		ddjvu_message_pop (ctx);
+}
+
 static gboolean
 djvu_document_load (EvDocument  *document,
 		    const char  *uri,
@@ -106,6 +157,7 @@ djvu_document_load (EvDocument  *document,
 	ddjvu_document_t *doc;
 	gchar *filename;
 	gboolean missing_files = FALSE;
+	GError *djvu_error = NULL;
 
 	/* FIXME: We could actually load uris  */
 	filename = g_filename_from_uri (uri, NULL, error);
@@ -124,8 +176,36 @@ djvu_document_load (EvDocument  *document,
 
 	djvu_document->d_document = doc;
 
-	while (!ddjvu_document_decoding_done (djvu_document->d_document)) 
-		djvu_handle_events (djvu_document, TRUE);
+	djvu_wait_for_message (djvu_document, DDJVU_DOCINFO, &djvu_error);
+	if (djvu_error) {
+		g_set_error_literal (error,
+				     EV_DOCUMENT_ERROR,
+				     EV_DOCUMENT_ERROR_INVALID,
+				     djvu_error->message);
+		g_error_free (djvu_error);
+		g_free (filename);
+		ddjvu_document_release (djvu_document->d_document);
+		djvu_document->d_document = NULL;
+
+		return FALSE;
+	}
+
+	if (ddjvu_document_decoding_error (djvu_document->d_document))
+		djvu_handle_events (djvu_document, TRUE, &djvu_error);
+
+	if (djvu_error) {
+		g_set_error_literal (error,
+				     EV_DOCUMENT_ERROR,
+				     EV_DOCUMENT_ERROR_INVALID,
+				     djvu_error->message);
+		g_error_free (djvu_error);
+		g_free (filename);
+		ddjvu_document_release (djvu_document->d_document);
+		djvu_document->d_document = NULL;
+		
+		return FALSE;
+	}
+	
 	g_free (djvu_document->uri);
 	djvu_document->uri = g_strdup (uri);
 
@@ -194,26 +274,37 @@ djvu_document_get_n_pages (EvDocument  *document)
 	return ddjvu_document_get_pagenum (djvu_document->d_document);
 }
 
+static void
+document_get_page_size (DjvuDocument *djvu_document,
+			gint          page,
+			double       *width,
+			double       *height)
+{
+	ddjvu_pageinfo_t info;
+	ddjvu_status_t r;
+	
+	while ((r = ddjvu_document_get_pageinfo(djvu_document->d_document, page, &info)) < DDJVU_JOB_OK)
+		djvu_handle_events(djvu_document, TRUE, NULL);
+	
+	if (r >= DDJVU_JOB_FAILED)
+		djvu_handle_events(djvu_document, TRUE, NULL);
+
+        *width = info.width * SCALE_FACTOR; 
+        *height = info.height * SCALE_FACTOR;
+}
+
 static void
 djvu_document_get_page_size (EvDocument   *document,
-			     int           page,
+			     EvPage       *page,
 			     double       *width,
 			     double       *height)
 {
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (document);
-        ddjvu_pageinfo_t info;
-	ddjvu_status_t r;
-	
+
 	g_return_if_fail (djvu_document->d_document);
 
-	while ((r = ddjvu_document_get_pageinfo(djvu_document->d_document, page, &info)) < DDJVU_JOB_OK)
-		djvu_handle_events(djvu_document, TRUE);
-	
-	if (r >= DDJVU_JOB_FAILED)
-		djvu_handle_events(djvu_document, TRUE);
-
-        *width = info.width * SCALE_FACTOR; 
-        *height = info.height * SCALE_FACTOR;
+	document_get_page_size (djvu_document, page->index,
+				width, height);
 }
 
 static cairo_surface_t *
@@ -231,10 +322,10 @@ djvu_document_render (EvDocument      *document,
 	double page_width, page_height, tmp;
 	static const cairo_user_data_key_t key;
 
-	d_page = ddjvu_page_create_by_pageno (djvu_document->d_document, rc->page);
+	d_page = ddjvu_page_create_by_pageno (djvu_document->d_document, rc->page->index);
 	
 	while (!ddjvu_page_decoding_done (d_page))
-		djvu_handle_events(djvu_document, TRUE);
+		djvu_handle_events(djvu_document, TRUE, NULL);
 
 	page_width = ddjvu_page_get_width (d_page) * rc->scale * SCALE_FACTOR + 0.5;
 	page_height = ddjvu_page_get_height (d_page) * rc->scale * SCALE_FACTOR + 0.5;
@@ -297,9 +388,6 @@ djvu_document_finalize (GObject *object)
 {
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (object);
 
-	if (djvu_document->search)
-	    djvu_text_free (djvu_document->search);
-
 	if (djvu_document->d_document)
 	    ddjvu_document_release (djvu_document->d_document);
 	    
@@ -346,6 +434,30 @@ djvu_document_document_iface_init (EvDocumentIface *iface)
 	iface->get_info = djvu_document_get_info;
 }
 
+static gchar *
+djvu_text_copy (DjvuDocument *djvu_document,
+		gint           page,
+		EvRectangle  *rectangle)
+{
+	miniexp_t page_text;
+	gchar    *text = NULL;
+
+	while ((page_text =
+		ddjvu_document_get_pagetext (djvu_document->d_document,
+					     page, "char")) == miniexp_dummy)
+		djvu_handle_events (djvu_document, TRUE, NULL);
+
+	if (page_text != miniexp_nil) {
+		DjvuTextPage *page = djvu_text_page_new (page_text);
+		
+		text = djvu_text_page_copy (page, rectangle);
+		djvu_text_page_free (page);
+		ddjvu_miniexp_release (djvu_document->d_document, page_text);
+	}
+
+	return text;
+}
+
 static gchar *
 djvu_selection_get_selected_text (EvSelection     *selection,
 				  EvRenderContext *rc,
@@ -364,7 +476,7 @@ djvu_selection_get_selected_text (EvSelection     *selection,
 	rectangle.x2 = points->x2 / SCALE_FACTOR;
 	rectangle.y2 = (height - points->y1) / SCALE_FACTOR;
 		
-      	text = djvu_text_copy (djvu_document, rc->page, &rectangle);
+      	text = djvu_text_copy (djvu_document, rc->page->index, &rectangle);
       
       	if (text == NULL)
 		text = g_strdup ("");
@@ -423,10 +535,10 @@ djvu_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document,
 	gdk_pixbuf_fill (pixbuf, 0xffffffff);
 	pixels = gdk_pixbuf_get_pixels (pixbuf);
 	
-	while (ddjvu_thumbnail_status (djvu_document->d_document, rc->page, 1) < DDJVU_JOB_OK)
-		djvu_handle_events(djvu_document, TRUE);
+	while (ddjvu_thumbnail_status (djvu_document->d_document, rc->page->index, 1) < DDJVU_JOB_OK)
+		djvu_handle_events(djvu_document, TRUE, NULL);
 		    
-	ddjvu_thumbnail_render (djvu_document->d_document, rc->page, 
+	ddjvu_thumbnail_render (djvu_document->d_document, rc->page->index, 
 				&thumb_width, &thumb_height,
 				djvu_document->thumbs_format,
 				gdk_pixbuf_get_rowstride (pixbuf), 
@@ -472,7 +584,7 @@ djvu_document_file_exporter_do_page (EvFileExporter  *exporter,
 {
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (exporter);
 	
-	g_string_append_printf (djvu_document->opts, "%d,", (rc->page) + 1); 
+	g_string_append_printf (djvu_document->opts, "%d,", (rc->page->index) + 1); 
 }
 
 static void
@@ -493,7 +605,7 @@ djvu_document_file_exporter_end (EvFileExporter *exporter)
 
 	ddjvu_job_t * job = ddjvu_document_print(djvu_document->d_document, fn, d_optc, d_optv);
 	while (!ddjvu_job_done(job)) {	
-		djvu_handle_events (djvu_document, TRUE);
+		djvu_handle_events (djvu_document, TRUE, NULL);
 	}
 
 	fclose(fn); 
@@ -536,108 +648,62 @@ djvu_document_init (DjvuDocument *djvu_document)
 	djvu_document->d_document = NULL;
 }
 
-static void
-djvu_document_find_begin (EvDocumentFind   *document,
-			  int               page,
-                          const char       *search_string,
-                          gboolean          case_sensitive)
+static GList *
+djvu_document_find_find_text (EvDocumentFind   *document,
+			      EvPage           *page,
+			      const char       *text,
+			      gboolean          case_sensitive)
 {
         DjvuDocument *djvu_document = DJVU_DOCUMENT (document);
+	miniexp_t page_text;
+	gdouble width, height;
+	GList *matches = NULL, *l;
 
-	if (djvu_document->search && 
-	    strcmp (search_string, djvu_text_get_text (djvu_document->search)) == 0)
-                return;
+	g_return_val_if_fail (text != NULL, NULL);
 
-        if (djvu_document->search)
-                djvu_text_free (djvu_document->search);
+	while ((page_text = ddjvu_document_get_pagetext (djvu_document->d_document,
+							 page->index,
+							 "char")) == miniexp_dummy)
+		djvu_handle_events (djvu_document, TRUE, NULL);
 
-        djvu_document->search = djvu_text_new (djvu_document,
-							  page,
-							  case_sensitive,
-							  search_string);
-}
-
-static int
-djvu_document_find_get_n_results (EvDocumentFind *document_find, int page)
-{
-	DjvuText *search = DJVU_DOCUMENT (document_find)->search;
-
-	if (search) {
-		return djvu_text_n_results (search, page);
-	} else {
-		return 0;
-	}
-}
-
-static gboolean
-djvu_document_find_get_result (EvDocumentFind *document_find,
-			       int             page,
-			       int             n_result,
-			       EvRectangle    *rectangle)
-{
-	DjvuDocument *djvu_document = DJVU_DOCUMENT (document_find);
-	DjvuText *search = djvu_document->search;
-	EvRectangle *r;
-	double width, height;
-
-	if (search == NULL)
-		return FALSE;
-
-	r = djvu_text_get_result (search, page, n_result);
-	if (r == NULL)
-		return FALSE;
-
-	djvu_document_get_page_size (EV_DOCUMENT (djvu_document), 
-		page, &width, &height);
-	rectangle->x1 = r->x1 * SCALE_FACTOR;
-	rectangle->y1 = height - r->y2 * SCALE_FACTOR;
-	rectangle->x2 = r->x2 * SCALE_FACTOR;
-	rectangle->y2 = height - r->y1 * SCALE_FACTOR;
+	if (page_text != miniexp_nil) {
+		DjvuTextPage *tpage = djvu_text_page_new (page_text);
 		
-	return TRUE;
-}
+		djvu_text_page_prepare_search (tpage, case_sensitive);
+		if (tpage->links->len > 0) {
+			djvu_text_page_search (tpage, text);
+			matches = tpage->results;
+		}
+		djvu_text_page_free (tpage);
+		ddjvu_miniexp_release (djvu_document->d_document, page_text);
+	}
 
-static int
-djvu_document_find_page_has_results (EvDocumentFind *document_find,
-				    int             page)
-{
-	DjvuText *search = DJVU_DOCUMENT (document_find)->search;
+	if (!matches)
+		return NULL;
 
-	return search && djvu_text_has_results (search, page);
-}
+	document_get_page_size (djvu_document, page->index, &width, &height);
+	for (l = matches; l && l->data; l = g_list_next (l)) {
+		EvRectangle *r = (EvRectangle *)l->data;
+		gdouble      tmp;
 
-static double
-djvu_document_find_get_progress (EvDocumentFind *document_find)
-{
-	DjvuText *search = DJVU_DOCUMENT (document_find)->search;
+		tmp = r->y1;
+		
+		r->x1 *= SCALE_FACTOR;
+		r->x2 *= SCALE_FACTOR;
+
+		tmp = r->y1;
+		r->y1 = height - r->y2 * SCALE_FACTOR;
+		r->y2 = height - tmp * SCALE_FACTOR;
+	}
 	
-	if (search == NULL) {
-		return 0;
-	}
 
-	return djvu_text_get_progress (search);
-}
-
-static void
-djvu_document_find_cancel (EvDocumentFind *document)
-{
-        DjvuDocument *djvu_document = DJVU_DOCUMENT (document);
-
-	if (djvu_document->search) {
-		djvu_text_free (djvu_document->search);
-		djvu_document->search = NULL;
-	}
+	return matches;
 }
 
 static void
 djvu_document_find_iface_init (EvDocumentFindIface *iface)
 {
-        iface->begin = djvu_document_find_begin;
-	iface->get_n_results = djvu_document_find_get_n_results;
-	iface->get_result = djvu_document_find_get_result;
-	iface->page_has_results = djvu_document_find_page_has_results;
-	iface->get_progress = djvu_document_find_get_progress;
-        iface->cancel = djvu_document_find_cancel;
+        iface->find_text = djvu_document_find_find_text;
 }
 
 static GList *
