@@ -26,13 +26,13 @@ extern void ev_marshal_VOID__ENUM_BOOLEAN (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* BOOLEAN:STRING (./ev-marshal.list:4) */
-extern void ev_marshal_BOOLEAN__STRING (GClosure     *closure,
-                                        GValue       *return_value,
-                                        guint         n_param_values,
-                                        const GValue *param_values,
-                                        gpointer      invocation_hint,
-                                        gpointer      marshal_data);
+/* VOID:STRING,STRING (./ev-marshal.list:4) */
+extern void ev_marshal_VOID__STRING_STRING (GClosure     *closure,
+                                            GValue       *return_value,
+                                            guint         n_param_values,
+                                            const GValue *param_values,
+                                            gpointer      invocation_hint,
+                                            gpointer      marshal_data);
 
 G_END_DECLS
 
