@@ -20,7 +20,8 @@
 #ifndef __EV_PAGE_CACHE_H__
 #define __EV_PAGE_CACHE_H__
 
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
+
 #include "ev-document.h"
 
 G_BEGIN_DECLS
@@ -54,6 +55,11 @@ void           ev_page_cache_get_height_to_page  (EvPageCache   *page_cache,
 						  gfloat         scale,
 						  gint          *height,
 						  gint	        *dual_height);
+void           ev_page_cache_get_thumbnail_size  (EvPageCache   *page_cache,
+						  gint           page,
+						  gint           rotation,
+						  gint          *width,
+						  gint          *height);
 gint           ev_page_cache_get_max_label_chars (EvPageCache   *page_cache);
 char          *ev_page_cache_get_page_label      (EvPageCache   *page_cache,
 						  gint           page);
