@@ -24,6 +24,7 @@
 #include "ev-view.h"
 #include "ev-pixbuf-cache.h"
 #include "ev-page-cache.h"
+#include "ev-jobs.h"
 #include "ev-image.h"
 #include "ev-form-field.h"
 #include "ev-selection.h"
@@ -96,8 +97,9 @@ struct _EvView {
 
 	EvDocument *document;
 
-	char *find_status;
-	int find_result;
+	/* Find */
+	GList **find_pages;
+	gint find_result;
 	gboolean jump_to_find_result;
 	gboolean highlight_find_results;
 	
@@ -184,7 +186,7 @@ struct _EvViewClass {
 	GtkLayoutClass parent_class;
 
 	void    (*binding_activated)	  (EvView         *view,
-					   EvScrollType   scroll,
+					   GtkScrollType   scroll,
 					   gboolean        horizontal);
 	void    (*zoom_invalid)		  (EvView         *view);
 	void    (*handle_link)		  (EvView         *view,
